#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2024/10/25 18:12
# @Author  : 兵
# @email    : 1747193328@qq.com
import os.path

from ase.io import read as ase_read
from ruamel.yaml import YAML

from NepTrain import module_path, utils, __version__
from .utils import check_env

def get_job_config(job_type):
    with open(os.path.join(module_path, "core/train/_template/job.yaml"), "r", encoding="utf8") as f:
        base_config = YAML().load(f)

    with open(os.path.join(module_path, f"core/train/_template/{job_type}.yaml"), "r", encoding="utf8") as f:
        type_config = YAML().load(f)
    job = utils.merge_yaml(base_config, type_config)
    return job
def init_template(argparse):
    if argparse.type=="bohrium":
        utils.print_tip("To use Bohrium, you must install the extra module with 'pip install dpdispatcher[bohrium]'.")


    if not argparse.force:
        utils.print_tip("For existing files, we choose to skip; if you need to forcibly generate and overwrite, please use -f or --force.")

    if not os.path.exists("./structure"):
        os.mkdir("./structure")
        utils.print_tip("Create the directory ./structure, please place the expanded structures that need to run MD into this folder!" )
    check_env()
    if not os.path.exists("./job.yaml") or argparse.force:
        utils.print_tip("You need to check and modify the vasp_job and vasp.cpu_core in the job.yaml file.")
        utils.print_warning("You also need to check and modify the settings for GPUMD active learning in job.yaml!")


        config = get_job_config(argparse.type)
        config["version"]=__version__
        if os.path.exists("train.xyz"):
            #检查下第一个结构有没有计算
            atoms=ase_read("./train.xyz",0,format="extxyz")

            if not (atoms.calc and "energy"   in atoms.calc.results):
                config["current_job"]="vasp"
                utils.print_warning("Check that the first structure in train.xyz has not been calculated; set the initial task to vasp!")
        else:
            utils.print_warning("Detected that there is no train.xyz in the current directory; please check the directory structure!")
            utils.print_tip("If there is a training set but the filename is not train.xyz, please unify the job.yaml.")


        with open("./job.yaml","w",encoding="utf8") as f:
            YAML().dump(config,f  )
    else:

        #已经存在 如果执行init  更新下
        base_config = get_job_config(argparse.type)

        with open("./job.yaml","r",encoding="utf8") as f:
            user_config = YAML().load(f)
        job=utils.merge_yaml(base_config,user_config)
        job["version"]=__version__

        with open("./job.yaml","w",encoding="utf8") as f:
            YAML().dump(job,f  )


    if not os.path.exists("./run.in")  or argparse.force:
        utils.print_tip("Create run.in; you can modify the ensemble settings! Temperature and time will be modified by the program!")

        utils.copy(os.path.join(module_path,"core/gpumd/run.in"),"./run.in")

    utils.print_success("Initialization is complete. After checking the files, you can run `NepTrain train job.yaml` to proceed.")



#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2025/8/2 18:23
# @Author  : 兵
# @email    : 1747193328@qq.com
import os
import re
from pathlib import Path

import numpy as np
from ase import Atoms
from ase.data import atomic_masses, atomic_numbers


def read_input_file(file_name: str) -> dict:
    input_key = {}
    if  not os.path.exists(file_name):
        return input_key
    pattern = r"^([A-Za-z_][A-Za-z0-9_]*)(?:\s+([^#\n]*?))?\s*(?:#.*)?$"
    with open(file_name, 'r', encoding="utf8") as f:
        # print(repr(f.read()))
        # groups = re.findall(r"^([A-Za-z_]+)(?:\s+([^#\n]*?))?\s*(?:#.*)?$", f.read(), re.MULTILINE)
        # for group in groups:
        #     input_key[group[0].strip()] = group[1].strip()
        for line in f:
            line = line.strip()
            if not line or line == "INPUT_PARAMETERS":
                continue
            m = re.match(pattern, line)
            if m:
                key, value = m.groups()
                input_key[key]= value.strip()

    return input_key



class StructureVar:
    pp_files={}
    orbs={}
    # masses={ symbol:atomic_masses[z] for symbol ,z in atomic_numbers.items()}
    @classmethod
    def init(cls,path):
        path = Path(path)
        upfs= path.glob("*.upf")
        for upf in upfs:
            try:
                with open( upf,"r") as f:
                    ufp_content = f.read()
                elem = re.search('element="(\w+)"',ufp_content).group(1)
                cls.pp_files[elem] = upf.name
            except:
                pass
        orbs= path.glob("*.orb")
        for orb in orbs:
            try:
                with open( orb,"r") as f:
                    orb_content = f.read()
                elem = re.search('Element\s+(\w+)',orb_content).group(1)
                cls.orbs[elem] = orb.name
            except:
                pass
        pass
    @classmethod
    def update(cls,structure):
        atom_names = structure["atom_names"]
        if "pp_files" in structure.data:
            for atom ,pp in zip(atom_names,structure["pp_files"]):
                StructureVar.pp_files[atom]=pp

        if "orb_files" in structure.data:
            for atom ,pp in zip(atom_names,structure["orb_files"]):
                StructureVar.orbs[atom]=pp
        # if "masses" in structure.data:
        #     for atom ,pp in zip(atom_names,structure["masses"]):
        #         StructureVar.masses[atom]=pp
    @classmethod
    def completion_abacus(cls,atoms:Atoms):

        atom_names =atoms.get_chemical_symbols()
        pp_files= {}
        orb_files= {}
        # masses=[]
        for atom in atom_names:
            if atom in cls.pp_files:
                pp_files[atom] = cls.pp_files[atom]
            if atom in cls.orbs:
                orb_files[atom] = cls.orbs[atom]

            # masses.append(cls.masses[atom])

        return pp_files,orb_files




#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2025/8/2 17:41
# @Author  : 兵
# @email    : 1747193328@qq.com
import math
import os.path

import numpy as np
from ase import Atoms
from ase.io import write as ase_write
from ase.calculators.abacus import Abacus, AbacusProfile

from NepTrain import utils, Config, module_path

from .io import read_input_file,StructureVar

atoms_index=1

@utils.iter_path_to_atoms(["*.vasp","*.xyz"],show_progress=True,
                 description="ABACUS calculation progress" )
def calculate_abacus(atoms:Atoms,argparse):
    global atoms_index
    StructureVar.init("./")

    if argparse.incar is not None and os.path.exists(argparse.incar):
        input_dict = read_input_file(argparse.incar)
    else:
        input_dict = read_input_file(os.path.join(module_path,"core/dft/abacus/INPUT"))
    directory=os.path.join(argparse.directory,f"{atoms_index}-{atoms.get_chemical_formula()}")
    atoms_index+=1
    command = f"{Config.get('environ','mpirun_path')} -n {argparse.n_cpu} {Config.get('environ','abacus_path',fallback='abacus')}"
    if "NEPTRAIN_ABACUS_COMMAND" in os.environ:
        command = os.environ["NEPTRAIN_ABACUS_COMMAND"]
    profile = AbacusProfile(command=command)


    a, b, c, alpha, beta, gamma = atoms.get_cell_lengths_and_angles()

    if argparse.kspacing is not None:
        input_dict["kspacing"]=argparse.kspacing

    pp_files,orb_files  =StructureVar.completion_abacus(atoms=atoms)


    calc = Abacus(profile=profile,
                  directory=directory,
                  pp=pp_files, basis=orb_files,
                  kpts=(math.ceil(argparse.ka[0]/a)  ,
                  math.ceil(argparse.ka[1]/b)  ,
                  math.ceil(argparse.ka[2]/c) ),
                  **input_dict
                  )


    calc.calculate(atoms, ('energy',"forces","stress"),None)
    atoms.calc = calc

    xx, yy, zz, yz, xz, xy = -calc.results['stress'] * atoms.get_volume()  # *160.21766
    atoms.info['virial'] = np.array([(xx, xy, xz), (xy, yy, yz), (xz, yz, zz)])
    # 这里没想好怎么设计config的格式化  就先使用原来的
    if "Config_type" not in atoms.info:
        atoms.info['Config_type'] = "NepTrain scf "
    atoms.info['Weight'] = 1.0
    del atoms.calc.results['stress']
    del atoms.calc.results['free_energy']

    return atoms



def run_abacus(argparse):
    result = calculate_abacus(argparse.model_path, argparse)
    path = os.path.dirname(argparse.out_file_path)
    if path and not os.path.exists(path):
        os.makedirs(path)
    if len(result) and isinstance(result[0], list):
        result = [atoms for _list in result for atoms in _list]
    ase_write(argparse.out_file_path, result, format="extxyz", append=argparse.append)

    utils.print_success("ABACUS calculation task completed!")




#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2024/10/29 19:56
# @Author  : 兵
# @email    : 1747193328@qq.com
import os.path


import numpy as np


from .utils import   get_dump_interval, calculate_angle, calculate_volume


def plot_md_selected(train_des,md_des,selected_des, save_path,decomposition="pca"):
    # 画一下图
    from matplotlib import pyplot as plt

    config = [
        # (文件名,图例,图例颜色)

    ]


    if md_des is not None:
        if isinstance(md_des,np.ndarray) and md_des.size!=0:
            config.append((md_des, 'new dataset', "#07cd66"))
        elif isinstance(md_des,list) :
            config.extend(md_des)
    if train_des is not None and train_des.size!=0:
        config.append((train_des, "base dataset","gray"))
    if selected_des is not None  and selected_des.size!=0:
        config.append((selected_des,'selected', "red"))

    fit_data = []

    for info in config:
        atoms_list_des= info[0]
        fit_data.append(atoms_list_des)
    if decomposition=="pca":
        from sklearn.decomposition import PCA

        reducer = PCA(n_components=2)
    else:
        from umap import UMAP

        reducer = UMAP(n_components=2)

    reducer.fit(np.vstack(fit_data))
    fig = plt.figure()
    for index, array in enumerate(fit_data):
        proj = reducer.transform(array)
        plt.scatter(proj[:, 0], proj[:, 1], label=config[index][1], c=config[index][2])
    leg_cols=len(config)//3 or 1
    plt.legend(ncols=leg_cols)
    # plt.axis('off')
    plt.savefig(save_path)
    plt.close(fig)









def plot_md_thermo(thermo_path ):
    from matplotlib import pyplot as plt






    fig = plt.figure()
    data = np.loadtxt(thermo_path)

    dump_interval = get_dump_interval()
    time = np.arange(0, len(data) * dump_interval / 1000, dump_interval / 1000)

    # read data
    temperature = data[:, 0]
    kinetic_energy = data[:, 1]
    potential_energy = data[:, 2]
    pressure_x = data[:, 3]
    pressure_y = data[:, 4]
    pressure_z = data[:, 5]

    num_columns = data.shape[1]

    if num_columns == 12:
        box_length_x = data[:, 9]
        box_length_y = data[:, 10]
        box_length_z = data[:, 11]
        volume = box_length_x * box_length_y * box_length_z
    elif num_columns == 18:
        ax, ay, az = data[:, 9], data[:, 10], data[:, 11]
        bx, by, bz = data[:, 12], data[:, 13], data[:, 14]
        cx, cy, cz = data[:, 15], data[:, 16], data[:, 17]

        a_vectors = np.column_stack((ax, ay, az))
        b_vectors = np.column_stack((bx, by, bz))
        c_vectors = np.column_stack((cx, cy, cz))

        box_length_x = np.sqrt(ax ** 2 + ay ** 2 + az ** 2)
        box_length_y = np.sqrt(bx ** 2 + by ** 2 + bz ** 2)
        box_length_z = np.sqrt(cx ** 2 + cy ** 2 + cz ** 2)

        box_angle_alpha = calculate_angle(b_vectors, c_vectors)
        box_angle_beta = calculate_angle(c_vectors, a_vectors)
        box_angle_gamma = calculate_angle(a_vectors, b_vectors)

        volume = calculate_volume(a_vectors, b_vectors, c_vectors)
    else:
        raise ValueError("Unsupported number of columns in thermo.out. Expected 12 or 18.")



    # Plot the data

    # set the color of the plot
    color_red = '#d22027'  # red
    color_blue = '#015699'  # blue
    color_yellow = '#fac00f'  # yellow

    # Subplot
    fig, axs = plt.subplots(2, 3, figsize=(12, 6), dpi=100)

    # Temperature
    axs[0, 0].plot(time, temperature, color=color_blue)
    axs[0, 0].set_xlabel('Time (ps)')
    axs[0, 0].set_ylabel('Temperature (K)')

    # Potential Energy and Kinetic Energy
    axs[0, 1].set_xlabel('Time (ps)')
    axs[0, 1].set_ylabel(r'Potential Energy ($x10^3$ eV)', color=color_red)
    axs[0, 1].plot(time, potential_energy / 1000, label='$P_E$', color=color_red)
    axs[0, 1].tick_params(axis='y', labelcolor=color_red, color=color_red)
    axs[0, 1].legend(frameon=False, bbox_to_anchor=(1.0, 0.4))

    axs_kinetic = axs[0, 1].twinx()
    axs_kinetic.set_ylabel('Kinetic Energy (eV)', color=color_blue)
    axs_kinetic.plot(time, kinetic_energy, label='$K_E$', color=color_blue)
    axs_kinetic.legend(frameon=False, bbox_to_anchor=(1.0, 0.3))
    axs_kinetic.tick_params(axis='y', labelcolor=color_blue, color=color_blue)

    # Pressure
    pressure_max = np.max(np.concatenate((pressure_x, pressure_y, pressure_z)))
    pressure_min = np.min(np.concatenate((pressure_x, pressure_y, pressure_z)))
    pressure_gap = pressure_max - pressure_min
    axs[0, 2].set_ylim(pressure_min - pressure_gap * 0.1, pressure_max + pressure_gap * 0.4)
    axs[0, 2].plot(time, pressure_x, label='Px', color=color_red)
    axs[0, 2].plot(time, pressure_y, label='Py', color=color_blue)
    axs[0, 2].plot(time, pressure_z, label='Pz', color=color_yellow)
    axs[0, 2].set_xlabel('Time (ps)')
    axs[0, 2].set_ylabel('Pressure (GPa)')
    axs[0, 2].legend(frameon=False, loc='upper right')

    # Lattice
    axs[1, 0].plot(time, box_length_x, label='a', color=color_red)
    axs[1, 0].plot(time, box_length_y, label='b', color=color_blue)
    axs[1, 0].plot(time, box_length_z, label='c', color=color_yellow)
    axs[1, 0].set_xlabel('Time (ps)')
    axs[1, 0].set_ylabel(r'Lattice Parameters (Å)')
    axs[1, 0].legend(frameon=False)

    # Volume
    axs[1, 1].plot(time, volume / 1000, label='Volume', color=color_blue)
    axs[1, 1].set_xlabel('Time (ps)')
    axs[1, 1].set_ylabel(r'Volume ($x10^3$ Å$^3$)')



    # Angles (only for triclinic systems)
    if num_columns == 18:
        axs[1, 2].plot(time, box_angle_alpha, label=r'$\alpha$', color=color_red)
        axs[1, 2].plot(time, box_angle_beta, label=r'$\beta$', color=color_blue)
        axs[1, 2].plot(time, box_angle_gamma, label=r'$\gamma$', color=color_yellow)
        axs[1, 2].set_xlabel('Time (ps)')
        axs[1, 2].set_ylabel(r'Angles ($\degree$)')
        axs[1, 2].legend(frameon=False)

    plt.tight_layout()

    plt.savefig(os.path.join(os.path.dirname(thermo_path),"thermo.png") , dpi=300)

    plt.close(fig)



#!/usr/bin/env python 
# -*- coding: utf-8 -*-
# @Time    : 2024/11/26 19:39
# @Author  : 兵
# @email    : 1747193328@qq.com
import os

import numpy as np


def calculate_angle(x, y):
    dot_product = np.einsum('ij,ij->i', x, y)
    norm_x = np.linalg.norm(x, axis=1)
    norm_y = np.linalg.norm(y, axis=1)
    angle_radians = np.arccos(dot_product / (norm_x * norm_y))
    return np.degrees(angle_radians)

def calculate_volume(a, b, c):
    volume = np.einsum('ij,ij->i', a, np.cross(b, c))
    return np.abs(volume)

# Determine dump_interval from run.in file
def get_dump_interval(run_in_file="run.in"):
    dump_interval = 10  # Default value
    if os.path.exists(run_in_file):
        with open(run_in_file, 'r') as file:
            for line in file:
                if "dump_thermo" in line:
                    try:
                        dump_interval = int(line.split()[1])
                        break
                    except (IndexError, ValueError):
                        pass
    return dump_interval